package BorderStyle::Test::Labeled;

use strict;
use warnings;

use Role::Tiny::With;
with 'BorderStyleRole::Source::Hash';

our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2022-02-14'; # DATE
our $DIST = 'BorderStyle'; # DIST
our $VERSION = '3.0.1'; # VERSION

our %BORDER = (
    v => 3,
    summary => 'A border style that uses labeled characters',
    utf8 => 1,
);

our %CHARS = (
    h_b => 'A',
    h_i => 'B',
    h_t => 'C',
    hd_i => 'D',
    hd_t => 'E',
    hu_b => 'F',
    hu_i => 'G',
    hv_i => 'H',
    ld_t => 'I',
    lu_b => 'J',
    lv_i => 'K',
    lv_r => 'L',
    rd_t => 'M',
    ru_b => 'N',
    rv_i => 'O',
    rv_l => 'P',
    v_i => 'Q',
    v_l => 'R',
    v_r => 'S',
);

1;
# ABSTRACT:

__END__

=pod

=encoding UTF-8

=head1 NAME

BorderStyle::Test::Labeled

=head1 VERSION

This document describes version 3.0.1 of BorderStyle::Test::Labeled (from Perl distribution BorderStyle), released on 2022-02-14.

=head1 DESCRIPTION

This border style uses a different label character for each border character.

    h_b => 'A',
    h_i => 'B',
    h_t => 'C',
    hd_i => 'D',
    hd_t => 'E',
    hu_b => 'F',
    hu_i => 'G',
    hv_i => 'H',
    ld_t => 'I',
    lu_b => 'J',
    lv_i => 'K',
    lv_r => 'L',
    rd_t => 'M',
    ru_b => 'N',
    rv_i => 'O',
    rv_l => 'P',
    v_i => 'Q',
    v_l => 'R',
    v_r => 'S',

=for Pod::Coverage ^(.+)$

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/BorderStyle>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-BorderStyle>.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTING


To contribute, you can send patches by email/via RT, or send pull requests on
GitHub.

Most of the time, you don't need to build the distribution yourself. You can
simply modify the code, then test via:

 % prove -l

If you want to build the distribution (e.g. to try to install it locally on your
system), you can install L<Dist::Zilla>,
L<Dist::Zilla::PluginBundle::Author::PERLANCAR>, and sometimes one or two other
Dist::Zilla plugin and/or Pod::Weaver::Plugin. Any additional steps required
beyond that are considered a bug and can be reported to me.

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2022, 2021, 2020 by perlancar <perlancar@cpan.org>.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=BorderStyle>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=cut
